@@ -27,6 +27,10 @@
 #include "fetchmail.h"
 
 #include "i18n.h"
+#if defined(HAVE_SETLOCALE) && defined(ENABLE_NLS) && defined(HAVE_STRFTIME)
+#include <time.h>
+#include <locale.h>
+#endif
 
 extern char *getenv();	/* needed on sysV68 R3V7.1. */
 
@@ -177,10 +181,18 @@ char *rfc822timestamp(void)
      * Conform to RFC822.  We generate a 4-digit year here, avoiding
      * Y2K hassles.  Max length of this timestamp in an English locale
      * should be 29 chars.  The only things that should vary by locale
-     * are the day and month abbreviations.
+     * are the day and month abbreviations.  The set_locale calls prevent
+     * weird multibyte i18n characters (such as kanji) for showing up
+     * in your Received headers.
      */
+#if defined(HAVE_SETLOCALE) && defined(ENABLE_NLS) && defined(HAVE_STRFTIME)
+    setlocale (LC_TIME, "C");
+#endif
     strftime(buf, sizeof(buf)-1, 
 	     "%a, %d %b %Y %H:%M:%S XXXXX (%Z)", localtime(&now));
+#if defined(HAVE_SETLOCALE) && defined(ENABLE_NLS) && defined(HAVE_STRFTIME)
+    setlocale (LC_TIME, "");
+#endif
     strncpy(strstr(buf, "XXXXX"), tzoffset(&now), 5);
 #else
     /*
