@@ -341,7 +341,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
 	 || ctl->server.authenticate == A_CRAM_MD5)
 	&& strstr(capabilities, "AUTH=CRAM-MD5"))
     {
-	if ((ok = do_cram_md5 (sock, "AUTHENTICATE", ctl)))
+	if ((ok = do_cram_md5 (sock, "AUTHENTICATE", ctl, NULL)))
 	{
 	    /* SASL cancellation of authentication */
 	    gen_send(sock, "*");
@@ -376,7 +376,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
 #ifdef NTLM_ENABLE
     if ((ctl->server.authenticate == A_ANY 
 	 || ctl->server.authenticate == A_NTLM) 
-	&& strstr (capabilities, "AUTH=NTLM"))
+	&& strstr (capabilities, "AUTH=NTLM")) {
 	if ((ok = do_imap_ntlm(sock, ctl)))
 	{
 	    /* SASL cancellation of authentication */
@@ -386,6 +386,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
 	}
 	else
 	    return(ok);
+    }
 #else
     if (ctl->server.authenticate == A_NTLM)
     {
