@@ -65,7 +65,7 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
     *bytes = *countp = *newp = -1;
 
     /* authenticate via CRAM-MD5 */
-    ok = do_cram_md5(sock, "AUTH", ctl);
+    ok = do_cram_md5(sock, "AUTH", ctl, "334 ");
     if (ok)
 	return(ok);
 
@@ -74,7 +74,7 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
      * canonical DNS name.
      */
     buf[0] = '\0';
-    for (qnp = ctl->smtphunt; qnp; qnp = qnp->next)
+    for (qnp = ctl->domainlist; qnp; qnp = qnp->next)
 	if (strlen(buf) + strlen(qnp->id) + 1 >= sizeof(buf))
 	    break;
 	else
@@ -125,13 +125,12 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
 
     /*
      * OK, if we got here it's time to become a pipe between the ODMR
-     * remote server (sending) and the local SMTP daemon (receiving).
-     * We're npt going to try to be a protocol machine; instead, we'll
-     * use select(2) to watch the read sides of both sockets and just
-     * throw their data at each other.
+     * remote server (sending) and the SMTP listener we've designated
+     * (receiving).  We're not going to try to be a protocol machine;
+     * instead, we'll use select(2) to watch the read sides of both
+     * sockets and just throw their data at each other.
      */
-    smtp_sock = SockOpen(ctl->smtphost, SMTP_PORT, NULL, NULL);
-    if (smtp_sock == -1)
+    if ((smtp_sock = smtp_open(ctl)) == -1)
 	return(PS_SOCKET);
     else
     {
@@ -180,6 +179,12 @@ static int odmr_getrange(int sock, struct query *ctl, const char *id,
     return(0);
 }
 
+static int odmr_logout(int sock, struct query *ctl)
+/* send logout command */
+{
+    return(gen_transact(sock, "QUIT"));
+}
+
 const static struct method odmr =
 {
     "ODMR",		/* ODMR protocol */
@@ -201,7 +206,7 @@ const static struct method odmr =
     NULL,		/* no way to fetch body */
     NULL,		/* no message trailer */
     NULL,		/* how to delete a message */
-    NULL,		/* log out, we're done */
+    odmr_logout,	/* log out, we're done */
     FALSE,		/* no, we can't re-poll */
 };
 
