@@ -14,11 +14,6 @@
 #include  <stdio.h>
 #include  <errno.h>
 #include  <string.h>
-#include  <signal.h>
-#include  <time.h>
-#ifdef HAVE_MEMORY_H
-#include  <memory.h>
-#endif /* HAVE_MEMORY_H */
 #if defined(STDC_HEADERS)
 #include  <stdlib.h>
 #endif
@@ -30,22 +25,16 @@
 #else
 #include  <varargs.h>
 #endif
-#include  <ctype.h>
 
 #include  "fetchmail.h"
 #include  "socket.h"
 #include  "smtp.h"
 #include  "i18n.h"
 
-/* BSD portability hack...I know, this is an ugly place to put it */
-#if !defined(SIGCHLD) && defined(SIGCLD)
-#define SIGCHLD	SIGCLD
-#endif
-
 /* makes the open_sink()/close_sink() pair non-reentrant */
 static int lmtp_responses;
 
-static int smtp_open(struct query *ctl)
+int smtp_open(struct query *ctl)
 /* try to open a socket to the appropriate SMTP server for this query */ 
 {
     /* maybe it's time to close the socket in order to force delivery */
@@ -71,7 +60,7 @@ static int smtp_open(struct query *ctl)
 	 * HELO name fails (RFC1123 section 5.2.5, paragraph 2).
 	 *
 	 * How we compute the true mailhost name to pass to the
-	 * listener doesn't affect behavior on RFC1123- violating
+	 * listener doesn't affect behavior on RFC1123-violating
 	 * listeners that check for name match; we're going to lose
 	 * on those anyway because we can never give them a name
 	 * that matches the local machine fetchmail is running on.
@@ -175,11 +164,6 @@ static int smtp_open(struct query *ctl)
 
 /* these are shared by open_sink and stuffline */
 static FILE *sinkfp;
-#ifndef HAVE_SIGACTION
-static RETSIGTYPE (*sigchld)(int);
-#else
-static struct sigaction sa_old;
-#endif /* HAVE_SIGACTION */
 
 int stuffline(struct query *ctl, char *buf)
 /* ship a line to the given control block's output sink (SMTP server or MDA) */
@@ -293,7 +277,11 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 	return(FALSE);
 
     /* our first duty is to keep the sacred foo counters turning... */
-    sprintf(boundary, 
+#ifdef HAVE_SNPRINTF
+    snprintf(boundary, sizeof(boundary),
+#else
+    sprintf(boundary,
+#endif /* HAVE_SNPRINTF */
 	    "foo-mani-padme-hum-%d-%d-%ld", 
 	    (int)getpid(), (int)getppid(), time((time_t *)NULL));
 
@@ -503,9 +491,6 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 /* set up sinkfp to be an input sink we can ship a message to */
 {
     struct	idlist *idp;
-#ifdef HAVE_SIGACTION
-    struct      sigaction sa_new;
-#endif /* HAVE_SIGACTION */
 
     *bad_addresses = *good_addresses = 0;
 
@@ -613,14 +598,24 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	 */
 	if (!msg->return_path[0])
 	{
-	    sprintf(addr, "%s@%s", ctl->remotename, ctl->server.truename);
+#ifdef HAVE_SNPRINTF
+	    snprintf(addr, sizeof(addr),
+#else
+	    sprintf(addr,
+#endif /* HAVE_SNPRINTF */
+		  "%s@%s", ctl->remotename, ctl->server.truename);
 	    ap = addr;
 	}
 	else if (strchr(msg->return_path, '@'))
 	    ap = msg->return_path;
 	else		/* in case Return-Path existed but was local */
 	{
-	    sprintf(addr, "%s@%s", msg->return_path, ctl->server.truename);
+#ifdef HAVE_SNPRINTF
+	    snprintf(addr, sizeof(addr),
+#else
+	    sprintf(addr,
+#endif /* HAVE_SNPRINTF */
+		    "%s@%s", msg->return_path, ctl->server.truename);
 	    ap = addr;
 	}
 
@@ -660,7 +655,6 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 		else
 		{
 		    char	errbuf[POPBUFSIZE];
-		    int res;
 
 #ifdef __UNUSED__
 		    /*
@@ -675,13 +669,19 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 		     * also as the body is discarded after calling
 		     * RSET!
 		     */
+		    int res;
 		    if ((res = handle_smtp_report(ctl, msg))==PS_REFUSED)
 			return(PS_REFUSED);
 #endif /* __UNUSED__ */
 
-		    strcpy(errbuf, idp->id);
+#ifdef HAVE_SNPRINTF
+		    snprintf(errbuf, sizeof(errbuf), "%s: %s",
+				    idp->id, smtp_response);
+#else
+		    strncpy(errbuf, idp->id, sizeof(errbuf));
 		    strcat(errbuf, ": ");
 		    strcat(errbuf, smtp_response);
+#endif /* HAVE_SNPRINTF */
 
 		    xalloca(from_responses[*bad_addresses], 
 			    char *, 
@@ -710,7 +710,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	if (!(*good_addresses)) 
 	{
 	    if (strchr(run.postmaster, '@'))
-		strcpy(addr, run.postmaster);
+		strncpy(addr, run.postmaster, sizeof(addr));
 	    else
 	    {
 #ifdef HAVE_SNPRINTF
@@ -915,15 +915,6 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 	    report(stderr, _("MDA open failed\n"));
 	    return(PS_IOERR);
 	}
-
-#ifndef HAVE_SIGACTION
-	sigchld = signal(SIGCHLD, SIG_DFL);
-#else
-	memset (&sa_new, 0, sizeof sa_new);
-	sigemptyset (&sa_new.sa_mask);
-	sa_new.sa_handler = SIG_DFL;
-	sigaction (SIGCHLD, &sa_new, &sa_old);
-#endif /* HAVE_SIGACTION */
     }
 
     /*
@@ -938,7 +929,7 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 void release_sink(struct query *ctl)
 /* release the per-message output sink, whether it's a pipe or SMTP socket */
 {
-    if (ctl->bsmtp)
+    if (ctl->bsmtp && sinkfp)
 	fclose(sinkfp);
     else if (ctl->mda)
     {
@@ -947,12 +938,6 @@ void release_sink(struct query *ctl)
 	    pclose(sinkfp);
 	    sinkfp = (FILE *)NULL;
 	}
-#ifndef HAVE_SIGACTION
-	signal(SIGCHLD, sigchld);
-#else
-	sigaction (SIGCHLD, &sa_old, NULL);
-#endif /* HAVE_SIGACTION */
-	deal_with_sigchld();
     }
 }
 
@@ -971,12 +956,6 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	}
 	else
 	    rc = 0;
-#ifndef HAVE_SIGACTION
-	signal(SIGCHLD, sigchld);
-#else
-	sigaction (SIGCHLD, &sa_old, NULL);
-#endif /* HAVE_SIGACTION */
-	deal_with_sigchld();
 	if (rc)
 	{
 	    report(stderr, 
@@ -984,7 +963,7 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 	    return(FALSE);
 	}
     }
-    else if (ctl->bsmtp)
+    else if (ctl->bsmtp && sinkfp)
     {
 	int error;
 
@@ -1167,7 +1146,11 @@ va_dcl
 #endif
     va_end(ap);
 
+#ifdef HAVE_SNPRINTF
+    snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\r\n");
+#else
     strcat(buf, "\r\n");
+#endif /* HAVE_SNPRINTF */
 
     stuffline(ctl, buf);
 }
